/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.plugin;

import static com.sos.scheduler.engine.kernel.util.Util.optional;
import static com.sos.scheduler.engine.kernel.util.XmlUtils.elementXPathOrNull;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.annotation.Nullable;

import org.w3c.dom.Element;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.sos.scheduler.engine.kernel.Scheduler;
import com.sos.scheduler.engine.kernel.scheduler.SchedulerException;
import com.sos.scheduler.engine.kernel.log.PrefixLog;
import com.sos.scheduler.engine.kernel.util.Lazy;

public class PluginReader {
    private static final String staticFactoryMethodName = "factory";

    private final PrefixLog log;
    private final Scheduler scheduler;
    private final Lazy<Injector> lazyInjector;

    PluginReader(PrefixLog log, Lazy<Injector> lazyInjector, Scheduler scheduler) {
        this.log = log;
        this.lazyInjector = lazyInjector;
        this.scheduler = scheduler;
    }

    @Nullable PluginAdapter tryReadPlugin(Element e) {
        String className = e.getAttribute("java_class");
        if (className.isEmpty())  throw new SchedulerException("Missing attribute java_class in <plugin>");
        return tryReadPlugin(className, elementXPathOrNull(e, "plugin.config"));
    }

    @Nullable private PluginAdapter tryReadPlugin(String className, Element elementOrNull) {
        try {
            return pluginAdapter(className, elementOrNull);
        } catch (Exception x) {
            PluginSubsystem.logError(log, "Plugin "+ className, x);
            return null;
        }
    }

    private PluginAdapter pluginAdapter(String className, Element elementOrNull) throws Exception {
        Plugin p = newPlugin(className, elementOrNull);
        return p instanceof CommandPlugin? new CommandPluginAdapter((CommandPlugin)p, className, log)
            : new PluginAdapter(p, className, log);
    }

    private Plugin newPlugin(String className, @Nullable Element elementOrNull) throws Exception {
        Class<? extends Plugin> c = pluginClassForName(className);
        Method factoryMethod = factoryMethodOrNull(c);
        return factoryMethod != null? newPluginByFactoryMethod(factoryMethod, elementOrNull)
                    : newPluginByDI(c, elementOrNull);
    }

    private Class<? extends Plugin> pluginClassForName(String name) throws ClassNotFoundException {
        Class<?> c = Class.forName(name);
        if (!Plugin.class.isAssignableFrom(c))
            throw new SchedulerException("Plugin does not implement " + Plugin.class.getName());

        @SuppressWarnings("unchecked")
        Class<? extends Plugin> result = (Class<? extends Plugin>)c;
        return result;
    }

    @Nullable private Method factoryMethodOrNull(Class<?> c) {
        try {
            return c.getMethod(staticFactoryMethodName);
        } catch (NoSuchMethodException x) {
            return null;
        }
    }

    private Plugin newPluginByFactoryMethod(Method m, Element elementOrNull) throws InvocationTargetException, IllegalAccessException {
        PluginFactory f = (PluginFactory)m.invoke(null);
        return f.newInstance(scheduler, elementOrNull);
    }

    private Plugin newPluginByDI(Class<? extends Plugin> c, @Nullable final Element elementOrNull) {
        Injector myInjector = lazyInjector.get().createChildInjector(Iterables.transform(optional(elementOrNull), elementToModule()));
        return myInjector.getInstance(c);
    }

    private static Function<Element,Module> elementToModule() {
        // Funktion, damit Abhängigkeit zu DI-Jars erst zur Laufzeit, bei Bedarf, verlangt werden.
        return new Function<Element,Module>() {
            @Override public Module apply(final Element e) {
                return new AbstractModule() {
                    @Override protected void configure() {
                        bind(Element.class).toInstance(e);
                    }
                };
            }
        };
    }
}
