/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_posix_mutex.cxx 13491 2008-04-16 10:16:11Z jz $

#include "zschimmer.h"
#include "z_posix.h"
#include "mutex.h"
#include "threads.h"


#ifdef Z_UNIX

#include <unistd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/socket.h>

#ifdef Z_SOLARIS
#   include <stropts.h>
#   include <sys/filio.h>       // ioctl( , FIONBIO )
#   define PTHREAD_MUTEX_FAST_NP  PTHREAD_MUTEX_RECURSIVE    // Solaris kennt PTHREAD_MUTEX_FAST_NP nicht
#endif

#ifdef Z_HPUX
#   define PTHREAD_MUTEX_FAST_NP PTHREAD_MUTEX_ERRORCHECK    // HP kennt PTHREAD_MUTEX_FAST_NP nicht
#endif

//-------------------------------------------------------------------------------------------------

namespace zschimmer {
namespace posix {

//--------------------------------------------------------------------------------------enter_mutex

void enter_mutex( pthread_mutex_t* mutex )
{
//fprintf(stderr,"pthread_mutex_lock(%p)\n", (void*)mutex );
    int err = pthread_mutex_lock( mutex );
    if( err )  throw_errno( err, "pthread_mutex_lock" );
}

//--------------------------------------------------------------------------------------leave_mutex

void leave_mutex( pthread_mutex_t* mutex )
{
//fprintf(stderr,"pthread_mutex_unlock(%p)\n", (void*)mutex );
    int err = pthread_mutex_unlock( mutex );
    if( err )  throw_errno( err, "pthread_mutex_unlock" );
}

//----------------------------------------------------------ndestroyable_mutex::Undestroyable_mutex

Undestroyable_mutex::Undestroyable_mutex( const string& name, Kind kind )
:
    Mutex_base(name,kind)
{
    int                 err;
    pthread_mutexattr_t attr;

    memset( &_system_mutex, 0, sizeof _system_mutex );

    err = pthread_mutexattr_init( &attr);
    if( err )  throw_xc( "pthread_mutexattr_init", _name.c_str() );


    int k = (int)PTHREAD_MUTEX_RECURSIVE;
    if( kind == kind_nonrecursive )
    {
//#       ifdef Z_DEBUG
            k = (int)PTHREAD_MUTEX_ERRORCHECK;
//#        else
//            kind = (int)PTHREAD_MUTEX_FAST_NP;
//#       endif
    }

    err = pthread_mutexattr_settype( &attr, k );
    if( err )  throw_xc( "pthread_mutexattr_settype", _name.c_str() );

    err = pthread_mutex_init( &_system_mutex, &attr );
    if( err )  throw_xc( "pthread_mutex_init", _name.c_str() );

    err = pthread_mutexattr_destroy( &attr );
    if( err )  throw_xc( "pthread_mutexattr_destroy", _name.c_str() );
}

//---------------------------------------------------------ndestroyable_mutex::~Undestroyable_mutex
    
Undestroyable_mutex::~Undestroyable_mutex()
{
    // Wird nicht zerstrt
}

//-----------------------------------------------------------------------Undestroyable_mutex::enter

void Undestroyable_mutex::enter()
{
    //if( !_dont_log )  fprintf(stderr,"pthread_mutex_lock %s (%p)\n", _name.c_str(), (void*)&_system_mutex );
    int err = pthread_mutex_lock( &_system_mutex );
    if( err )  throw_errno( err, "pthread_mutex_lock", _name.c_str() );
}

//-------------------------------------------------------------------Undestroyable_mutex::try_enter

bool Undestroyable_mutex::try_enter()
{
    //if( !_dont_log )  fprintf(stderr,"pthread_mutex_lock try %s (%p)\n", _name.c_str(), (void*)&_system_mutex );
    int err = pthread_mutex_trylock( &_system_mutex );
    if( err ) 
    {
        if( err == EBUSY )
        {
            //if( !_dont_log )  fprintf(stderr,"pthread_mutex_lock BUSY %s (%p)\n", _name.c_str(), (void*)&_system_mutex );
            return false;
        }

        //fprintf(stderr,"pthread_mutex_lock %s (%p) ERROR=%d\n", _name.c_str(), (void*)&_system_mutex, err );
        throw_errno( err, "pthread_mutex_lock", _name.c_str() );
    }

    return true;
}

//-----------------------------------------------------------------------Undestroyable_mutex::leave

void Undestroyable_mutex::leave()
{
    //if( !_dont_log )  fprintf(stderr,"pthread_mutex_unlock %s (%p)\n", _name.c_str(), (void*)&_system_mutex );
    int err = pthread_mutex_unlock( &_system_mutex );
    if( err ) 
    {
        //fprintf(stderr,"pthread_mutex_unlock %s (%p) ERROR=%d\n", _name.c_str(), (void*)&_system_mutex, err );
        throw_errno( err, "pthread_mutex_unlock", _name.c_str() );
    }
}

//-----------------------------------------------------------Undestroyable_mutex::locking_thread_id

Thread_id Undestroyable_mutex::locking_thread_id()
{
    return 0;   // Sollte die Thread-Id des sperrenden Threads sein.
}

//-------------------------------------------------------------------------------------utex::~Mutex

Mutex::~Mutex()
{
    int err = pthread_mutex_destroy( &_system_mutex );
    if( err )  Z_LOG( Z_FUNCTION << " ERROR pthread_mutex_destroy() err=" << err << " " << strerror( err ) << '\n' );
}

//-------------------------------------------------------------------------------------------------

} //namespace posix
} //namespace zschimmer

#endif
